32-bit compat ioctl for inotify in 2.6.11-mm2

(cf. http://blog.blackdown.de/2005/03/11/32-bit-compat-ioctl-for-inotify/)

Signed-off-by: Juergen Kreileder <jk@blackdown.de>

--- linux-2.6.11-mm2/fs/inotify.c	2005-03-11 01:23:42.000000000 +0100
+++ linux/fs/inotify.c	2005-03-11 02:07:05.000000000 +0100
@@ -33,6 +33,7 @@
 #include <linux/list.h>
 #include <linux/writeback.h>
 #include <linux/inotify.h>
+#include <linux/compat.h>
 
 #include <asm/ioctls.h>
 
@@ -920,8 +921,8 @@ static int inotify_ignore(struct inotify
 	return 0;
 }
 
-static int inotify_ioctl(struct inode *ip, struct file *file, unsigned int cmd,
-			 unsigned long arg)
+static long inotify_ioctl(struct file *file, unsigned int cmd,
+			  unsigned long arg)
 {
 	struct inotify_device *dev;
 	struct inotify_watch_request request;
@@ -959,13 +960,68 @@ static int inotify_ioctl(struct inode *i
 	return ret;
 }
 
+#ifdef CONFIG_COMPAT
+struct inotify_watch_request_compat {
+	compat_uptr_t	name;	/* filename name */
+	__u32		mask;	/* event mask */
+};
+#define INOTIFY_WATCH_COMPAT	\
+	_IOR(INOTIFY_IOCTL_MAGIC, 1, struct inotify_watch_request_compat)
+
+static long inotify_ioctl_compat(struct file *file, unsigned int cmd,
+				 unsigned long arg)
+{
+	struct inotify_device *dev;
+	struct inotify_watch_request request;
+	struct inotify_watch_request_compat request_compat;
+	void __user *p;
+	int ret = -ENOTTY;
+	s32 wd;
+
+	dev = file->private_data;
+	p = compat_ptr(arg);
+
+	get_inotify_dev(dev);
+
+	switch (cmd) {
+	case INOTIFY_WATCH_COMPAT:
+		if (unlikely(copy_from_user(&request_compat, p,
+					    sizeof (request_compat)))) {
+			ret = -EFAULT;
+			break;
+		}
+		request.name = compat_ptr(request_compat.name);
+		request.mask = request_compat.mask;
+		ret = inotify_add_watch(dev, &request);
+		break;
+	case INOTIFY_IGNORE:
+		if (unlikely(copy_from_user(&wd, p, sizeof (wd)))) {
+			ret = -EFAULT;
+			break;
+		}
+		ret = inotify_ignore(dev, wd);
+		break;
+	case FIONREAD:
+		ret = put_user(dev->queue_size, (int __user *) p);
+		break;
+	}
+
+	put_inotify_dev(dev);
+
+	return ret;
+}
+#endif
+
 static struct file_operations inotify_fops = {
 	.owner		= THIS_MODULE,
 	.poll		= inotify_poll,
 	.read		= inotify_read,
 	.open		= inotify_open,
 	.release	= inotify_release,
-	.ioctl		= inotify_ioctl,
+	.unlocked_ioctl	= inotify_ioctl,
+#ifdef CONFIG_COMPAT
+	.compat_ioctl	= inotify_ioctl_compat,
+#endif
 };
 
 static struct miscdevice inotify_device = {
=
